"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var audio_object_1 = require("./audio-object");
var sin = Math.sin, PI = Math.PI, asin = Math.asin, atan = Math.atan, tan = Math.tan;
var Oscillator = /** @class */ (function () {
    function Oscillator(_a) {
        var frequency = _a.frequency, _b = _a.phase, phase = _b === void 0 ? 0 : _b, _c = _a.amplitude, amplitude = _c === void 0 ? 1 : _c, _d = _a.type, type = _d === void 0 ? 'sine' : _d;
        this._frequency = frequency;
        this._amplitude = audio_object_1.toAudioObject(amplitude);
        this._phase = audio_object_1.toAudioObject(phase);
        this._type = type;
    }
    Oscillator.prototype.frequency = function (value) {
        if (value === undefined)
            return this._frequency;
        this._frequency = value;
    };
    Oscillator.prototype.amplitude = function (value) {
        if (value === undefined)
            return this._amplitude;
        this._amplitude = audio_object_1.toAudioObject(value);
    };
    Oscillator.prototype.phase = function (value) {
        if (value === undefined)
            return this._phase;
        this._phase = audio_object_1.toAudioObject(value);
    };
    Oscillator.prototype.type = function (value) {
        if (value === undefined)
            return this._type;
        this._type = value;
    };
    Oscillator.prototype.tf = function (t) {
        switch (this._type) {
            case 'sine':
                return (this._amplitude.tf(t) *
                    sin(2 * PI * t * this._frequency + this._phase.tf(t)));
            case 'square':
                return (this._amplitude.tf(t) *
                    (sin(2 * PI * t * this._frequency + this._phase.tf(t)) > 0 ? 1 : -1));
            case 'triangle':
                var time = asin(sin(2 * PI * this._frequency * t + this._phase.tf(t)));
                time /= PI / 2;
                return time * this._amplitude.tf(t);
            case 'sawthoot':
                return (-((2 * this._amplitude.tf(t)) / PI) *
                    atan(1 / tan(t * PI * this._frequency + PI / 2 + this._phase.tf(t) / 2)));
            default:
                throw new Error('Oscillator type not recognized');
        }
    };
    return Oscillator;
}());
exports.Oscillator = Oscillator;
